@@ -2,6 +2,7 @@
 from __future__ import annotations  # allow pre-definition use of types
 from dataclasses import dataclass
 from enum import Enum
+from functools import cached_property
 from math import ceil
 from typing import Type
 
@@ -45,11 +46,12 @@ class Side(str, Enum):
 
     @property
     def other_side(self) -> Side:
-        # use a lookup to ensure failure with unknown side
+        # use a lookup to ensure KeyError with unknown value
         return {Side.HEDGE: Side.LONG, Side.LONG: Side.HEDGE}[self]
 
     @classmethod
     def from_string(cls, side_string: str) -> Side:
+        # use a lookup to ensure KeyError with unknown value
         return {'hedge': cls.HEDGE, 'long': cls.LONG}[side_string.lower()]
 
 
@@ -58,7 +60,7 @@ class NominalOracleUnitsXSatsPerBch(int):
         super().__init__()
         validators.instance(value, int)  # i.e. don't allow silent coercion
         validators.less_equal(self, SCRIPT_INT_MAX_WHEN_JAVASCRIPT)
-        validators.greater_equal(self, SATS_PER_BCH)  # equivalent of 1 nominal oracle unit
+        validators.greater_equal(self, SATS_PER_BCH)  # i.e. minimum is 1 nominal oracle unit
 
 
 class LongLeverage(float):
@@ -84,6 +86,7 @@ class RedemptionType(str, Enum):
 
     @classmethod
     def from_string(cls, redemption_type_string: str) -> RedemptionType:
+        # use a lookup to ensure KeyError with unknown value
         return {'liquidation': cls.LIQUIDATION, 'maturation': cls.MATURATION}[redemption_type_string.lower()]
 
 
@@ -111,22 +114,48 @@ class ContractProposal:
         self.validate()
 
     ###############
-    # Calculated key values used in actual contract creation and execution
+    # Parameterized Input Values
     ###############
-    @property
-    def total_input_sats(self) -> UtxoSats:
-        return UtxoSats(ceil(self.nominal_oracleUnits_x_satsPerBch / self.long_liquidation_price_oracleUnits_per_bch))
+    @cached_property
+    def _input_sats_lookup(self) -> dict[(Role | None, Side | None), UtxoSats | None]:
+        # Total input sats is basically the definition of an AnyHedge contract
+        total_input_sats = UtxoSats(ceil(self.nominal_oracleUnits_x_satsPerBch / self.long_liquidation_price_oracleUnits_per_bch))
 
-    @property
-    def hedge_input_sats(self) -> UtxoSats:
+        # Hedge input sats defines the starting point
         # Round gets a potentially slightly more accurate number.
         # The precision to zero-sum comes from defining long position as "everything else needed to cover liquidation".
-        return UtxoSats(round_half_up(self.nominal_oracleUnits_x_satsPerBch / self.start_price_oracleUnits_per_bch))
+        hedge_input_sats = UtxoSats(round_half_up(self.nominal_oracleUnits_x_satsPerBch / self.start_price_oracleUnits_per_bch))
 
-    @property
-    def long_input_sats(self) -> UtxoSats:
-        return UtxoSats(self.total_input_sats - self.hedge_input_sats)
+        # Long input sats is just everything needed to cover the rest of the total
+        long_input_sats = UtxoSats(total_input_sats - hedge_input_sats)
 
+        return {
+            (None,       None):       total_input_sats,
+            (None,       Side.HEDGE): hedge_input_sats,
+            (None,       Side.LONG):  long_input_sats,
+            (Role.MAKER, None):       hedge_input_sats if self.maker_side == Side.HEDGE else long_input_sats,
+            (Role.MAKER, Side.HEDGE): hedge_input_sats if self.maker_side == Side.HEDGE else None,
+            (Role.MAKER, Side.LONG):  long_input_sats  if self.maker_side == Side.LONG  else None,
+            (Role.TAKER, None):       hedge_input_sats if self.taker_side == Side.HEDGE else long_input_sats,
+            (Role.TAKER, Side.HEDGE): hedge_input_sats if self.taker_side == Side.HEDGE else None,
+            (Role.TAKER, Side.LONG):  long_input_sats  if self.taker_side == Side.LONG  else None,
+        }
+
+    def input_sats(self, role: Role | None = None, side: Side | None = None) -> UtxoSats:
+        key = (role, side)
+        value = self._input_sats_lookup[key]
+        if value is None:
+            raise ValueError(f'mismatch of role and side query ({key}) with actual contract roles (maker={self.maker_side})')
+        return value
+
+    def input_oracleUnits(self, role: Role | None = None, side: Side | None = None) -> OracleUnit:
+        unit = self.oracle_unit_cls
+        bch = self.input_sats(side=side, role=role).bch
+        return unit(bch * float(self.start_price_oracleUnits_per_bch))
+
+    ###############
+    # Derivative values
+    ###############
     @property
     def oracle_unit_cls(self) -> Type[OracleUnit]:
         return oracle_pubkey_to_unit_class[self.oracle_public_key]
@@ -136,66 +165,63 @@ class ContractProposal:
         # Until two-sided leverage is enabled in AnyHedge, this is always None
         return None
 
-    ###############
-    # Approximate unit versions of the inputs
-    ###############
-    @property
-    def total_input_oracleUnits(self) -> OracleUnit:
-        return self.oracle_unit_cls(self.effective_nominal_oracleUnits)
-
-    @property
-    def hedge_input_oracleUnits(self) -> OracleUnit:
-        return self.oracle_unit_cls(self.nominal_oracleUnits_x_satsPerBch / SATS_PER_BCH)
-
-    @property
-    def long_input_oracleUnits(self) -> OracleUnit:
-        return self.oracle_unit_cls(self.long_input_sats.bch * float(self.start_price_oracleUnits_per_bch))
-
-    ###############
-    # Reverse calculated effective values from contract perspective
-    ###############
     @property
     def duration_seconds(self) -> int:
         return self.maturity_timestamp - self.start_timestamp
 
-    @property
-    def effective_nominal_oracleUnits(self) -> OracleUnit:
-        return self.oracle_unit_cls(self.nominal_oracleUnits_x_satsPerBch / SATS_PER_BCH)
-
     @property
     def effective_long_leverage(self) -> LongLeverage:
         return LongLeverage(1 / (1 - self.long_liquidation_price_oracleUnits_per_bch / self.start_price_oracleUnits_per_bch))
 
-    ###############
-    # Sided views
-    ###############
     @property
     def taker_side(self) -> Side:
         return self.maker_side.other_side
 
+    ###############
+    # Property access to input sats
+    ###############
     @property
-    def maker_input_sats(self) -> Sats:
-        if self.maker_side == Side.HEDGE:
-            return self.hedge_input_sats
-        return self.long_input_sats
+    def total_input_sats(self) -> UtxoSats:
+        return self.input_sats()
 
     @property
-    def taker_input_sats(self) -> Sats:
-        if self.taker_side == Side.HEDGE:
-            return self.hedge_input_sats
-        return self.long_input_sats
+    def hedge_input_sats(self) -> UtxoSats:
+        return self.input_sats(side=Side.HEDGE)
+
+    @property
+    def long_input_sats(self) -> UtxoSats:
+        return self.input_sats(side=Side.LONG)
+
+    @property
+    def maker_input_sats(self) -> UtxoSats:
+        return self.input_sats(role=Role.MAKER)
+
+    @property
+    def taker_input_sats(self) -> UtxoSats:
+        return self.input_sats(role=Role.TAKER)
+
+    ###############
+    # Property access to unit conversions of inputs
+    ###############
+    @property
+    def total_input_oracleUnits(self) -> OracleUnit:
+        return self.input_oracleUnits()
+
+    @property
+    def hedge_input_oracleUnits(self) -> OracleUnit:
+        return self.input_oracleUnits(side=Side.HEDGE)
+
+    @property
+    def long_input_oracleUnits(self) -> OracleUnit:
+        return self.input_oracleUnits(side=Side.LONG)
 
     @property
     def maker_input_oracleUnits(self) -> OracleUnit:
-        if self.maker_side == Side.HEDGE:
-            return self.hedge_input_oracleUnits
-        return self.long_input_oracleUnits
+        return self.input_oracleUnits(role=Role.MAKER)
 
     @property
     def taker_input_oracleUnits(self) -> OracleUnit:
-        if self.taker_side == Side.HEDGE:
-            return self.hedge_input_oracleUnits
-        return self.long_input_oracleUnits
+        return self.input_oracleUnits(role=Role.TAKER)
 
     ###############
     # Constructors
@@ -240,21 +266,21 @@ class ContractFunding:
     fee_agreements: tuple[FeeAgreement, ...]
 
     @property
-    def fee_sats_to_maker(self):
+    def fee_sats_to_maker(self) -> Sats:
         return aggregate_fee_sats_to_role(self.fee_agreements, Role.MAKER)
 
     @property
-    def fee_sats_to_taker(self):
+    def fee_sats_to_taker(self) -> Sats:
         return aggregate_fee_sats_to_role(self.fee_agreements, Role.TAKER)
 
     @property
-    def fee_sats_to_hedge(self):
+    def fee_sats_to_hedge(self) -> Sats:
         if self.base_proposal.maker_side == Side.HEDGE:
             return self.fee_sats_to_maker
         return self.fee_sats_to_taker
 
     @property
-    def fee_sats_to_long(self):
+    def fee_sats_to_long(self) -> Sats:
         if self.base_proposal.maker_side == Side.LONG:
             return self.fee_sats_to_maker
         return self.fee_sats_to_taker
@@ -263,25 +289,25 @@ class ContractFunding:
     # Unit value calculations
     ###############
     @property
-    def fee_oracleUnits_to_maker(self):
-        fee_bch = aggregate_fee_sats_to_role(self.fee_agreements, Role.MAKER).bch
+    def fee_oracleUnits_to_maker(self) -> OracleUnit:
+        fee_bch = self.fee_sats_to_maker.bch
         fee_oracleUnits = self.base_proposal.oracle_unit_cls(fee_bch * float(self.base_proposal.start_price_oracleUnits_per_bch))
         return fee_oracleUnits
 
     @property
-    def fee_oracleUnits_to_taker(self):
-        fee_bch = aggregate_fee_sats_to_role(self.fee_agreements, Role.TAKER).bch
+    def fee_oracleUnits_to_taker(self) -> OracleUnit:
+        fee_bch = self.fee_sats_to_taker.bch
         fee_oracleUnits = self.base_proposal.oracle_unit_cls(fee_bch * float(self.base_proposal.start_price_oracleUnits_per_bch))
         return fee_oracleUnits
 
     @property
-    def fee_oracleUnits_to_hedge(self):
+    def fee_oracleUnits_to_hedge(self) -> OracleUnit:
         if self.base_proposal.maker_side == Side.HEDGE:
             return self.fee_oracleUnits_to_maker
         return self.fee_oracleUnits_to_taker
 
     @property
-    def fee_oracleUnits_to_long(self):
+    def fee_oracleUnits_to_long(self) -> OracleUnit:
         if self.base_proposal.maker_side == Side.LONG:
             return self.fee_oracleUnits_to_maker
         return self.fee_oracleUnits_to_taker
@@ -317,7 +343,7 @@ class ContractFunding:
         # 1/pA = (u - pS(fP - fR)) / (u pS)
         # pA = u pS / (u - pS(fP - fR))
 
-        u = self.base_proposal.effective_nominal_oracleUnits
+        u = self.base_proposal.hedge_input_oracleUnits
         pS = float(self.base_proposal.start_price_oracleUnits_per_bch)
         total_fees_to_role_bch = direction * aggregate_fee_sats_to_role(self.fee_agreements, role).bch
         adjusted_price_oracleUnits_per_bch = u * pS / (u - pS * total_fees_to_role_bch)
