@@ -1,5 +1,6 @@
 # Built-in imports
 from __future__ import annotations
+from codecs import decode
 from dataclasses import dataclass
 from functools import cached_property
 from typing import (
@@ -259,16 +260,22 @@ def parse_oracle_price_message(msg_hex: str) -> tuple[ScriptTimestamp, int, int,
     [16, 24) price sequence (4 bytes)
     [24, 32) price (4 bytes)
     """
-    if len(msg_hex) == 32:
-        # might be a price message
-        pass
-    else:
-        print(f'unknown oracle message. probably meta: {msg_hex}')
+    if len(msg_hex) != 32:
+        raise ValueError(f'wrong byte-length ({len(msg_hex)/2}) for a price message (16)')
+
+    # timestamp, msg_sequence, price_sequence/metadatatype should always parse and be valid
+    timestamp = ScriptTimestamp(_parse_as_little_endian_signed_int(msg_hex[0:8]))
+    msg_sequence = _parse_as_little_endian_signed_int(msg_hex[8:16])
+    price_sequence = _parse_as_little_endian_signed_int(msg_hex[16:24])
+
+    if price_sequence == 0:
+        raise RuntimeError('this message is out of specification with price sequence == 0')
+    elif price_sequence < 0:
+        raise ValueError('this is a metadata message, not a price message')
+
+    # Price should always parse and be valid if we get to this point. Unspecified error if not.
+    price = ScriptPriceInOracleUnitsPerBch(_parse_as_little_endian_signed_int(msg_hex[24:]))
 
-    timestamp = ScriptTimestamp(int(_swap_hex_endianness(msg_hex[0:8]), 16))
-    msg_sequence = int(_swap_hex_endianness(msg_hex[8:16]), 16)
-    price_sequence = int(_swap_hex_endianness(msg_hex[16:24]), 16)
-    price = ScriptPriceInOracleUnitsPerBch(int(_swap_hex_endianness(msg_hex[24:]), 16))
     return timestamp, msg_sequence, price_sequence, price
 
 
@@ -277,3 +284,9 @@ def _swap_hex_endianness(data_hex: str) -> str:
     data_bytes.reverse()
     data_hex_reversed = ''.join(format(x, '02x') for x in data_bytes)
     return data_hex_reversed
+
+
+def _parse_as_little_endian_signed_int(data_hex: str) -> int:
+    data_bin = decode(data_hex, 'hex_codec')
+    value = int.from_bytes(data_bin, byteorder='little', signed=True)
+    return value
