@@ -80,6 +80,7 @@ DERIVED_LONG_INPUT_SATS = UtxoSats(14_283_878)
 DERIVED_HEDGE_INPUT_ORACLE_UNITS = UsdEM2Beta(100_00.0)
 # for long, it's DERIVED_LONG_INPUT_SATS * START_PRICE_USDEM2_PER_BCH / SATS_PER_BCH
 DERIVED_LONG_INPUT_ORACLE_UNITS = UsdEM2Beta(2856.7756)
+DERIVED_TOTAL_INPUT_ORACLE_UNITS = UsdEM2Beta(DERIVED_HEDGE_INPUT_ORACLE_UNITS + DERIVED_LONG_INPUT_ORACLE_UNITS)
 
 
 def standard_contract_proposal(
@@ -133,14 +134,23 @@ class TestContractProposalDerivedValues(unittest.TestCase):
     ######################
     # Reverse calculated intent values from the contract's perspective
     ######################
-    def test_effective_nominal_oracleUnits(self):
-        self.assertAlmostEqual(standard_contract_proposal().effective_nominal_oracleUnits, NOMINAL_USDEM2, delta=.001)
-
     def test_effective_long_leverage(self):
         self.assertAlmostEqual(standard_contract_proposal().effective_long_leverage, LONG_LEVERAGE, delta=.001)
 
     ######################
-    # Sided versions of values
+    # Unit values
+    ######################
+    def test_total_input_oracleUnits(self):
+        self.assertEqual(standard_contract_proposal().total_input_oracleUnits, DERIVED_TOTAL_INPUT_ORACLE_UNITS)
+
+    def test_hedge_input_oracleUnits(self):
+        self.assertEqual(standard_contract_proposal().hedge_input_oracleUnits, DERIVED_HEDGE_INPUT_ORACLE_UNITS)
+
+    def test_long_input_oracleUnits(self):
+        self.assertEqual(standard_contract_proposal().long_input_oracleUnits, DERIVED_LONG_INPUT_ORACLE_UNITS)
+
+    ######################
+    # Role versions of values
     ######################
     def test_all_sided_values(self):
         hedge_maker_contract = standard_contract_proposal(maker_side=Side.HEDGE)
@@ -159,6 +169,22 @@ class TestContractProposalDerivedValues(unittest.TestCase):
         self.assertEqual(long_maker_contract.maker_input_oracleUnits, DERIVED_LONG_INPUT_ORACLE_UNITS)
         self.assertEqual(long_maker_contract.taker_input_oracleUnits, DERIVED_HEDGE_INPUT_ORACLE_UNITS)
 
+    ######################
+    # Role versions of values
+    ######################
+    def test_parameterized_lookup_error_on_invalid_combinations(self):
+        maker_hedge_contract = standard_contract_proposal(maker_side=Side.HEDGE)
+        with self.assertRaises(ValueError):
+            maker_hedge_contract.input_sats(role=Role.MAKER, side=Side.LONG)
+        with self.assertRaises(ValueError):
+            maker_hedge_contract.input_sats(role=Role.TAKER, side=Side.HEDGE)
+
+        taker_hedge_contract = standard_contract_proposal(maker_side=Side.LONG)
+        with self.assertRaises(ValueError):
+            taker_hedge_contract.input_sats(role=Role.TAKER, side=Side.LONG)
+        with self.assertRaises(ValueError):
+            taker_hedge_contract.input_sats(role=Role.MAKER, side=Side.HEDGE)
+
 
 class TestContractProposalValidation(unittest.TestCase):
     def test_ValueError_if_duration_too_short(self):
@@ -196,7 +222,7 @@ class TestContractFundingDerivedValues(unittest.TestCase):
         # See the implementation for explanation of the formula
         # base values
         funding = standard_contract_funding()
-        u = funding.base_proposal.effective_nominal_oracleUnits
+        u = funding.base_proposal.hedge_input_oracleUnits
         pS = float(funding.base_proposal.start_price_oracleUnits_per_bch)
 
         # in the context of long taker
@@ -225,7 +251,7 @@ class TestContractFundingDerivedValues(unittest.TestCase):
     def test_effective_adjusted_price_when_maker_is_long(self):
         # base values
         funding = standard_contract_funding(standard_contract_proposal(maker_side=Side.LONG))
-        u = funding.base_proposal.effective_nominal_oracleUnits
+        u = funding.base_proposal.hedge_input_oracleUnits
         pS = float(funding.base_proposal.start_price_oracleUnits_per_bch)
 
         # in the context of hedge taker
